
local DurationLoggedOff
local currentWSGTime = nil
local currentABTime = nil
local currentAVTime = nil
local UIrealWSG = "Unqueued"
local UIrealAB = "Unqueued"
local UIrealAV = "Unqueued"
local inBG = nil
local skipWSG
local skipAB
local skipAV

local addonName = GetAddOnMetadata("RealQueues", "Title");

local RealQueues = CreateFrame("Frame","RealQueueFrame",UIParent,"TranslucentFrameTemplate")
RealQueues:SetMovable(true)
RealQueues:EnableMouse(true)
RealQueues:RegisterForDrag("LeftButton")
RealQueues:SetScript("OnDragStart", RealQueues.StartMoving)
RealQueues:SetScript("OnDragStop", RealQueues.StopMovingOrSizing)
RealQueues:SetPoint("CENTER");
RealQueues:SetSize(185,100)
RealQueues:Hide()

local WSGDispText = RealQueues:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
WSGDispText:SetPoint("TOPLEFT",20,-20)
local ABDispText = RealQueues:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
ABDispText:SetPoint("LEFT",20,0)
local AVDispText = RealQueues:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
AVDispText:SetPoint("BOTTOMLEFT",20,20)
WSGDispText:SetText("|cff50945eReal WSG: Unqueued")
ABDispText:SetText("|cff948350Real AB: Unqueued")
AVDispText:SetText("|cff506e94Real AV: Unqueued")
RealQueues:RegisterEvent("UPDATE_BATTLEFIELD_STATUS")
RealQueues:RegisterEvent("PLAYER_ENTERING_WORLD")
RealQueues:RegisterEvent("PLAYER_LOGOUT")
RealQueues:RegisterEvent("ADDON_LOADED")
RealQueues:RegisterEvent("PLAYER_LOGIN")



function UIToggleCheck()
    if(showUI == 1) then
        RealQueues:Show()
    elseif(showUI == 0) then
        RealQueues:Hide()
    end
end

function DrawRQMinimapIcon()
	LibStub("LibDBIcon-1.0"):Register("RealQueues", LibStub("LibDataBroker-1.1"):NewDataObject("RealQueues",
	{
		type = "data source",
		text = addonName,
		icon = "Interface\\Icons\\Ability_Spy",
        OnClick = function(self, button) 
                if(RealQueues:IsShown()) then
                    showUI = 0
                    UIToggleCheck()
                return
                elseif(not(RealQueues:IsShown())) then
                    showUI = 1
                    UIToggleCheck()
                return end
			--end
		end,
		OnTooltipShow = function(tooltip)
			tooltip:AddDoubleLine(format("%s", addonName), format("|cff7f7f7fv%s", GetAddOnMetadata(addonName, "Version")));
			tooltip:AddLine("|cff7f7f7fby Reziarfg|r");
			tooltip:AddLine("|cFFCFCFCFLeft Click: Toggle RealQueues UI");
		end
	}));
end

DrawRQMinimapIcon()


local function updateUI(queueStatus,paused,isWSGQueued,isABQueued,isAVQueued) --Update the UI for latest data, no modification of underlying data
    if(queueStatus == "queued") then
        UIToggleCheck()
        UIrealWSG = SecondsToTime((realWSG/1000))
        UIrealAB = SecondsToTime((realAB/1000))
        UIrealAV = SecondsToTime((realAV/1000))
        if(paused) then
            if(isWSGQueued == true) then
                UIWSG = "|cff838383Real WSG: "..UIrealWSG
            elseif(isWSGQueued == false) then
                UIWSG = "|cff838383Real WSG: Unqueued"
            end
            if(isABQueued == true) then
                UIAB = "|cff838383Real AB: "..UIrealAB
            elseif(isABQueued == false) then
                UIAB = "|cff838383Real AB: Unqueued"
            end
            if(isAVQueued == true) then
                UIAV = "|cff838383Real AV: "..UIrealAV
            elseif(isAVQueued == false) then
                UIAV = "|cff838383Real AV: Unqueued"
            end
            WSGDispText:SetText(UIWSG);
            ABDispText:SetText(UIAB);
            AVDispText:SetText(UIAV);
        elseif(not(paused)) then
            if(isWSGQueued == true) then
                UIWSG = "|cff50945eReal WSG: "..UIrealWSG
            elseif(isWSGQueued == false) then
                UIWSG = "|cff50945eReal WSG: Unqueued"
            end
            if(isABQueued == true) then
                UIAB = "|cff948350Real AB: "..UIrealAB
            elseif(isABQueued == false) then
                UIAB = "|cff948350Real AB: Unqueued"
            end
            if(isAVQueued == true) then
                UIAV = "|cff506e94Real AV: "..UIrealAV
            elseif(isAVQueued == false) then
                UIAV = "|cff506e94Real AV: Unqueued"
            end
            WSGDispText:SetText(UIWSG);
            ABDispText:SetText(UIAB);
            AVDispText:SetText(UIAV);
        end


    elseif(queueStatus == "unqueued") then
        showUI = 0
        WSGDispText:SetText("|cff50945eReal WSG: Unqueued")
        ABDispText:SetText("|cff948350Real AB: Unqueued")
        AVDispText:SetText("|cff506e94Real AV: Unqueued")
        UIToggleCheck()
    end
end
    

local function checkClearedQueue(paused)--Iterates through BGs to identify if any/all queues have been unqueued, happens 3 times per update event
    local allBGs = {}
    local WSGReset = 1
    local ABReset = 1
    local AVReset = 1
    local WSGQueued = true
    local ABQueued = true
    local AVQueued = true
    for i=1, MAX_BATTLEFIELD_QUEUES do
        local status, mapName, teamSize, unknownBGvar, registeredMatch, suspendedQueue, queueType, gameType, role, asGroup, shortDescription, longDescription = GetBattlefieldStatus(i)
        if(mapName == nil) then
        table.insert(allBGs,"Vacant")
        elseif(mapName ~= nil) then
            table.insert(allBGs,mapName)
        end
    end
    for key, value in pairs(allBGs) do
        if value == "Warsong Gulch" then
            WSGReset = 0
            WSGQueued = true
        elseif value == "Arathi Basin" then
            ABReset = 0
            ABQueued = true
        elseif value == "Alterac Valley" then
            AVReset = 0
            AVQueued = true
        end
    end
    if(WSGReset == 1) then
        realWSG = 0
        WSGQueued = false
    end
    if(ABReset == 1) then
        realAB = 0
        ABQueued = false
    end
    if(AVReset == 1) then
        realAV = 0
        AVQueued = false
    end
   if (WSGReset ~= 1 or ABReset ~= 1 or AVReset ~= 1) then
        updateUI("queued",paused,WSGQueued,ABQueued,AVQueued)
    elseif (WSGReset == 1 and ABReset == 1 and AVReset == 1) then
        updateUI("unqueued",paused,WSGQueued,ABQueued,AVQueued)
    end    
end



local function updateBG(bgName,battleFieldIndex,paused)
    if (bgName == "Warsong Gulch" and skipWSG == 1) then
        skipWSG = 0
        return
    elseif (bgName == "Arathi Basin" and skipAB == 1) then
        skipAB = 0
        return
    elseif (bgName == "Alterac Valley" and skipAV == 1) then
        skipAV = 0
        return
    end
    updateUI("queued",paused)
    checkClearedQueue(paused)
    if(bgName == "Warsong Gulch") then
        if(realWSG == 0) then
            realWSG = GetBattlefieldTimeWaited(battleFieldIndex);
            newWSGTime = nil
            WSGStartTime = GetTime()
            return
        end
        if(realWSG > 0) then
            currentWSGTime = GetTime()
            if (newWSGTime == nil) then
                newWSGTime = currentWSGTime - WSGStartTime
                WSGiter = newWSGTime --initializes first iteration time
                if(not(paused)) then realWSG = realWSG + (newWSGTime*1000) end
                return
            elseif (newWSGTime ~= nil) then
                newWSGTime = currentWSGTime - WSGStartTime --difference between current time and start time
                local WSGdiff = newWSGTime - WSGiter --finds difference between previous iteration time and initial time
                WSGiter = newWSGTime
                if(not(paused)) then realWSG = realWSG + (WSGdiff*1000) end               
            end
        end
    elseif(bgName == "Arathi Basin") then
        if(realAB == 0) then
            realAB = GetBattlefieldTimeWaited(battleFieldIndex);
            newABTime = nil
            ABStartTime = GetTime()
            return
        end
        if(realAB > 0) then
            currentABTime = GetTime()
            if (newABTime == nil) then
                newABTime = currentABTime - ABStartTime
                ABiter = newABTime
                if(not(paused)) then realAB = realAB + (newABTime*1000) end
                return
            elseif (newABTime ~= nil) then
                newABTime = currentABTime - ABStartTime
                local ABdiff = newABTime - ABiter
                ABiter = newABTime
                if(not(paused)) then realAB = realAB + (ABdiff*1000) end
            end
        end
    elseif(bgName == "Alterac Valley") then
    if(realAV == 0) then
        realAV = GetBattlefieldTimeWaited(battleFieldIndex);
        newAVTime = nil
        AVStartTime = GetTime()
        return
    end
    if(realAV > 0) then
        currentAVTime = GetTime()
        if (newAVTime == nil) then
            newAVTime = currentAVTime - AVStartTime
            AViter = newAVTime
            if(not(paused)) then realAV = realAV + (newAVTime*1000) end
            return
        elseif (newAVTime ~= nil) then
            newAVTime = currentAVTime - AVStartTime
            local AVdiff = newAVTime - AViter
            AViter = newAVTime
            if(not(paused)) then realAV = realAV + (AVdiff*1000) end
        end
    end
    end
    updateUI("queued",paused)
    checkClearedQueue(paused)
end

local function checkSuspended(mapName,battleFieldIndex)--Iterates through all BGs, looking for a suspended queue to then pass to updateBG which will pause timers
    for i=1, MAX_BATTLEFIELD_QUEUES do
        local status2, mapName2, teamSize2, unknownBGvar2, registeredMatch2, suspendedQueue2, queueType2, gameType2, role2, asGroup2, shortDescription2, longDescription2 = GetBattlefieldStatus(i);
        if(suspendedQueue2 == true) then
            updateBG(mapName,battleFieldIndex,true);
            return
        end
    end
    updateBG(mapName,battleFieldIndex,false);
end

local function checkANYQueued(mapName,battleFieldIndex)--Makes sure there aren't at least one queued BG before clearing the queue
    for i=1, MAX_BATTLEFIELD_QUEUES do
        local status2, mapName2, teamSize2, unknownBGvar2, registeredMatch2, suspendedQueue2, queueType2, gameType2, role2, asGroup2, shortDescription2, longDescription2 = GetBattlefieldStatus(i);
        if(status2 ~= "none") then--If a queue is anything but "none", eg:"Warsong Gulch"
            return
        end
    end
    checkSuspended(mapName,battleFieldIndex)
end

local function loginCheck()--run only on logon to determine if any queues exist, to allow clearing out any that were lost on logoff
    for i=1, MAX_BATTLEFIELD_QUEUES do
        local status2, mapName2, teamSize2, unknownBGvar2, registeredMatch2, suspendedQueue2, queueType2, gameType2, role2, asGroup2, shortDescription2, longDescription2 = GetBattlefieldStatus(i);
        if(suspendedQueue2 == true) then
            checkClearedQueue(true);
            return
        end
    end
    checkClearedQueue(false);
end


local waitTable = {};
local waitFrame = nil;

local function RealQueues_Wait(delay, func, ...)--A delay function to be used sparingly until I figure out how to perform some necessary checks, especially on logon
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end


local function EventHandler(self, event, battleFieldIndex)
    if(event =="ADDON_LOADED") then--When saved variables load
        UIToggleCheck()
    end
    if(event == "PLAYER_LOGIN") then
        --RealQueues_Wait(15,checkANYQueued)--Waits 15 seconds on . This is a result of UPDATE_BATTLEFIELD_STATUS not triggering between the logoff and logon and the saved variables being retained between sessions and starting the count back where it left off even after timeout. Better way to address this?
        if(RealQueues_LogoffTime ~= nil) then DurationLoggedOff = (GetTime()-RealQueues_LogoffTime) end
        if(realWSG ~= nil and realWSG > 0) then WSGiter = WSGiter + DurationLoggedOff end
        if(realAB ~= nil and realAB > 0) then ABiter = ABiter + DurationLoggedOff end--;print("added",DurationLoggedOff,"to",SecondsToTime(ABiter)) end
        if(realAV ~= nil and realAV > 0) then AViter = AViter + DurationLoggedOff end
        RealQueues_Wait(10,loginCheck)
    end
    if(event == "PLAYER_LOGOUT") then
        RealQueues_LogoffTime = GetTime()
    end
    if(event == "PLAYER_ENTERING_WORLD") then--Happens on zoneins
        if(realWSG == nil) then realWSG = 0; end --What if the saved variables haven't loaded yet? Do we know player_entering_world always triggers after addon_loaded(with vars? (It seems to always happen after addon_loaded))
        if(realAB == nil) then realAB = 0; end
        if(realAV == nil) then realAV = 0; end
        if(realWSG > 0) then
            skipWSG = 1
        end
        if(realAB > 0) then
            skipAB = 1
        end
        if(realAV > 0) then
            skipAV = 1
        end
    end
    if event == "UPDATE_BATTLEFIELD_STATUS" then
            local status, mapName, teamSize, unknownBGvar, registeredMatch, suspendedQueue, queueType, gameType, role, asGroup, shortDescription, longDescription = GetBattlefieldStatus(battleFieldIndex);
            if (InActiveBattlefield() == true) then
                updateBG(mapName,battleFieldIndex,true);
                return
            end
            if ((InActiveBattlefield() == false) and (status == "queued")) then
                checkSuspended(mapName,battleFieldIndex)
            elseif ((InActiveBattlefield() == false) and (status == "none")) then--This should only trigger on leaving a queue. Checks if there are no active queues, before the queue can clear
                checkANYQueued(mapName,battleFieldIndex)--checkclearedqueue?
            end
    end

end

RealQueues:SetScript("OnEvent", EventHandler)

